@@ -3,14 +3,15 @@ use crate::error::Result;
 use crate::list::Grouping;
 use crate::path;
 use crate::query::{EditQuery, ListQuery, RangeQuery, ScriptOrDirQuery, ScriptQuery};
-use crate::script_type::ScriptType;
+use crate::script_type::{ScriptFullType, ScriptType};
 use crate::tag::TagFilter;
 use crate::Either;
 use serde::Serialize;
+use std::num::NonZeroUsize;
 use std::path::PathBuf;
 use structopt::clap::AppSettings::{
-    self, AllArgsOverrideSelf, AllowExternalSubcommands, AllowLeadingHyphen, DisableHelpFlags,
-    DisableHelpSubcommand, DisableVersion, Hidden, TrailingVarArg,
+    self, AllArgsOverrideSelf, AllowExternalSubcommands, AllowLeadingHyphen, ColoredHelp,
+    DisableHelpFlags, DisableHelpSubcommand, DisableVersion, Hidden, TrailingVarArg,
 };
 use structopt::StructOpt;
 
@@ -24,6 +25,7 @@ use help_str::*;
 pub use types::*;
 
 const NO_FLAG_SETTINGS: &[AppSettings] = &[
+    ColoredHelp,
     AllowLeadingHyphen,
     DisableHelpFlags,
     TrailingVarArg,
@@ -88,7 +90,7 @@ pub struct RootArgs {
 }
 
 #[derive(StructOpt, Debug, Serialize)]
-#[structopt(settings = &[AllowLeadingHyphen, AllArgsOverrideSelf])]
+#[structopt(global_setting = ColoredHelp, settings = &[AllowLeadingHyphen, AllArgsOverrideSelf])]
 pub struct Root {
     #[structopt(skip = false)]
     #[serde(skip)]
@@ -148,7 +150,7 @@ impl AliasRoot {
 }
 
 #[derive(StructOpt, Debug, Serialize)]
-#[structopt(settings = &[AllArgsOverrideSelf])]
+#[structopt(settings = &[AllArgsOverrideSelf, ColoredHelp])]
 pub enum Subs {
     #[structopt(external_subcommand)]
     Other(Vec<String>),
@@ -162,7 +164,7 @@ pub enum Subs {
     Help { args: Vec<String> },
     #[structopt(setting = Hidden, about = "Print the help message of env variables")]
     EnvHelp {
-        #[structopt(default_value = "-")]
+        #[structopt(default_value = "-", help = SCRIPT_QUERY_HELP)]
         script_query: ScriptQuery,
     },
     #[structopt(setting = Hidden)]
@@ -172,14 +174,14 @@ pub enum Subs {
     #[structopt(about = "Edit hyper script", settings = &[AllowLeadingHyphen, TrailingVarArg])]
     Edit {
         #[structopt(long, short = "T", help = TYPE_HELP)]
-        ty: Option<ScriptType>,
+        ty: Option<ScriptFullType>,
         #[structopt(long, short)]
         no_template: bool,
         #[structopt(long, short, help = TAGS_HELP)]
         tags: Option<TagFilter>,
         #[structopt(long, help = "Create script without invoking the editor")]
         fast: bool,
-        #[structopt(default_value = "?", help = NEW_ANONYMOUS_HELP)]
+        #[structopt(default_value = "?", help = EDIT_QUERY_HELP)]
         edit_query: EditQuery<ScriptQuery>,
         content: Vec<String>,
     },
@@ -216,24 +218,24 @@ pub enum Subs {
         error_no_previous: bool,
         #[structopt(long, short, requires = "previous-args", help = "")]
         dir: Option<PathBuf>,
-        #[structopt(default_value = "-")]
+        #[structopt(default_value = "-", help = SCRIPT_QUERY_HELP)]
         script_query: ScriptQuery,
         #[structopt(help = "Command line args to pass to the script")]
         args: Vec<String>,
     },
     #[structopt(about = "Execute the script query and get the exact file")]
     Which {
-        #[structopt(default_value = "-")]
+        #[structopt(default_value = "-", help = SCRIPT_QUERY_HELP)]
         script_query: ScriptQuery,
     },
     #[structopt(about = "Print the script to standard output")]
     Cat {
-        #[structopt(default_value = "-")]
+        #[structopt(default_value = "-", help = SCRIPT_QUERY_HELP)]
         script_query: ScriptQuery,
     },
     #[structopt(about = "Remove the script")]
     RM {
-        #[structopt(required = true, min_values = 1)]
+        #[structopt(required = true, min_values = 1, help = LIST_QUERY_HELP)]
         queries: Vec<ListQuery>,
         #[structopt(
             long,
@@ -249,8 +251,9 @@ pub enum Subs {
     CP {
         #[structopt(long, short, help = TAGS_HELP)]
         tags: Option<TagFilter>,
-        origin: ScriptQuery,
-        #[structopt(help = NEW_ANONYMOUS_HELP)]
+        #[structopt(help = SCRIPT_QUERY_HELP)]
+        origin: ListQuery,
+        #[structopt(help = EDIT_CONCRETE_QUERY_HELP)]
         new: EditQuery<ScriptOrDirQuery>,
     },
     #[structopt(about = "Move the script to another one")]
@@ -259,8 +262,9 @@ pub enum Subs {
         ty: Option<ScriptType>,
         #[structopt(long, short, help = TAGS_HELP)]
         tags: Option<TagFilter>,
+        #[structopt(help = LIST_QUERY_HELP)]
         origin: ListQuery,
-        #[structopt(help = NEW_ANONYMOUS_HELP)]
+        #[structopt(help = EDIT_CONCRETE_QUERY_HELP)]
         new: Option<EditQuery<ScriptOrDirQuery>>,
     },
     #[structopt(about = "Manage script tags")]
@@ -276,7 +280,7 @@ pub enum Subs {
 pub enum History {
     RM {
         // TODO: dir
-        #[structopt(required = true, min_values = 1)]
+        #[structopt(required = true, min_values = 1, help = LIST_QUERY_HELP)]
         queries: Vec<ListQuery>,
         range: RangeQuery,
     },
@@ -289,7 +293,7 @@ pub enum History {
     #[structopt(about = "Humble an event by it's id")]
     Humble { event_id: u64 },
     Show {
-        #[structopt(default_value = "-")]
+        #[structopt(default_value = "-", help = LIST_QUERY_HELP)]
         queries: Vec<ListQuery>,
         #[structopt(short, long, default_value = "10")]
         limit: u32,
@@ -301,7 +305,7 @@ pub enum History {
         dir: Option<PathBuf>,
     },
     Neglect {
-        #[structopt(required = true, min_values = 1)]
+        #[structopt(required = true, min_values = 1, help = LIST_QUERY_HELP)]
         queries: Vec<ListQuery>,
     },
     #[structopt( settings = NO_FLAG_SETTINGS)]
@@ -311,7 +315,7 @@ pub enum History {
         args: Vec<String>,
     },
     Tidy {
-        #[structopt(required = true, min_values = 1)]
+        #[structopt(required = true, min_values = 1, help = LIST_QUERY_HELP)]
         queries: Vec<ListQuery>,
     },
 }
@@ -324,12 +328,15 @@ pub struct List {
     pub long: bool,
     #[structopt(long, possible_values(&["tag", "tree", "none"]), default_value = "tag", help = "Grouping style.")]
     pub grouping: Grouping,
+    #[structopt(long, help = "Limit the amount of scripts found.")]
+    pub limit: Option<NonZeroUsize>,
     #[structopt(long, help = "No color and other decoration.")]
     pub plain: bool,
     #[structopt(long, help = "Show file path to the script.", conflicts_with = "long")]
     pub file: bool,
     #[structopt(long, help = "Show name of the script.", conflicts_with = "long")]
     pub name: bool,
+    #[structopt(help = LIST_QUERY_HELP)]
     pub queries: Vec<ListQuery>,
 }
 
@@ -352,7 +359,8 @@ fn print_help<S: AsRef<str>>(cmds: impl IntoIterator<Item = S>) -> Result {
             return Ok(());
         }
     }
-    clap.clone().print_help()?;
+    let mut clap = clap.clone().setting(ColoredHelp);
+    clap.print_help()?;
     println!();
     std::process::exit(0);
 }
@@ -400,8 +408,9 @@ impl Root {
     pub fn sanitize(&mut self) -> Result {
         match &mut self.subcmd {
             Some(Subs::Other(args)) => {
-                let args =
-                    std::array::IntoIter::new(["hs", "run"]).chain(args.iter().map(|s| s.as_str()));
+                let args = ["hs", "run"]
+                    .into_iter()
+                    .chain(args.iter().map(|s| s.as_str()));
                 self.subcmd = Some(Subs::from_iter(args));
                 log::info!("執行模式 {:?}", self.subcmd);
             }
