@@ -1,5 +1,5 @@
 use super::help_str::*;
-use crate::script_type::ScriptType;
+use crate::script_type::ScriptFullType;
 use serde::Serialize;
 use structopt::clap::AppSettings::AllowLeadingHyphen;
 use structopt::StructOpt;
@@ -15,21 +15,25 @@ pub struct Types {
 pub enum TypesSubs {
     #[structopt(external_subcommand)]
     Other(Vec<String>),
-    LS,
+    LS {
+        #[structopt(long)]
+        show_sub: bool,
+    },
     Template {
         #[structopt(long, short)]
         edit: bool,
         #[structopt(help = TYPE_HELP)]
-        ty: ScriptType,
+        ty: ScriptFullType,
     },
 }
 
 impl Types {
     pub fn sanitize(&mut self) {
         match self.subcmd.as_ref() {
-            None => self.subcmd = Some(TypesSubs::LS),
+            None => self.subcmd = Some(TypesSubs::LS { show_sub: false }),
             Some(TypesSubs::Other(args)) => {
-                let args = std::array::IntoIter::new(["types", "template"])
+                let args = ["types", "template"]
+                    .into_iter()
                     .chain(args.iter().map(|s| s.as_str()));
                 self.subcmd = Some(TypesSubs::from_iter(args));
             }
