@@ -3,17 +3,25 @@ use crate::config::{Config, PromptLevel};
 use crate::error::{Error, Result};
 use crate::fuzzy;
 use crate::script::ScriptInfo;
-use crate::script_repo::{RepoEntry, ScriptRepo};
+use crate::script_repo::{RepoEntry, ScriptRepo, Visibility};
 use crate::util::{get_display_type, hijack_ctrlc_once};
 use crate::SEP;
 use fxhash::FxHashSet as HashSet;
 
+fn compute_vis(bang: bool) -> Visibility {
+    if bang {
+        Visibility::All
+    } else {
+        Visibility::Normal
+    }
+}
+
 pub async fn do_list_query<'a>(
     repo: &'a mut ScriptRepo,
     queries: &[ListQuery],
 ) -> Result<Vec<RepoEntry<'a>>> {
     if queries.is_empty() {
-        return Ok(repo.iter_mut(false).collect());
+        return Ok(repo.iter_mut(Visibility::Normal).collect());
     }
     let mut mem = HashSet::<i64>::default();
     let mut ret = vec![];
@@ -31,9 +39,9 @@ pub async fn do_list_query<'a>(
         // SAFETY: `mem` 已保證回傳的陣列不可能包含相同的資料
         let repo = unsafe { &mut *repo_ptr };
         match query {
-            ListQuery::Pattern(re, og) => {
+            ListQuery::Pattern(re, og, bang) => {
                 let mut is_empty = true;
-                for script in repo.iter_mut(false) {
+                for script in repo.iter_mut(compute_vis(*bang)) {
                     if re.is_match(&script.name.key()) {
                         is_empty = false;
                         insert!(script);
@@ -68,11 +76,14 @@ pub async fn do_script_query<'b>(
     forbid_prompt: bool,
 ) -> Result<Option<RepoEntry<'b>>> {
     log::debug!("開始尋找 `{:?}`", script_query);
-    let all = script_query.bang;
+    let mut visibility = compute_vis(script_query.bang);
+    if finding_filtered {
+        visibility = visibility.invert();
+    }
     match &script_query.inner {
         ScriptQueryInner::Prev(prev) => {
             assert!(!finding_filtered); // XXX 很難看的作法，應設法靜態檢查
-            let latest = script_repo.latest_mut(*prev, all);
+            let latest = script_repo.latest_mut(*prev, visibility);
             log::trace!("找最新腳本");
             return if latest.is_some() {
                 Ok(latest)
@@ -80,24 +91,15 @@ pub async fn do_script_query<'b>(
                 Err(Error::Empty)
             };
         }
-        ScriptQueryInner::Exact(name) => {
-            if finding_filtered {
-                Ok(script_repo.get_hidden_mut(name))
-            } else {
-                Ok(script_repo.get_mut(name, all))
-            }
-        }
+        ScriptQueryInner::Exact(name) => Ok(script_repo.get_mut(name, visibility)),
         ScriptQueryInner::Fuzz(name) => {
             let level = if forbid_prompt {
                 PromptLevel::Never
             } else {
                 Config::get_prompt_level()
             };
-            let iter = if finding_filtered {
-                script_repo.iter_hidden_mut()
-            } else {
-                script_repo.iter_mut(all)
-            };
+
+            let iter = script_repo.iter_mut(visibility);
             let fuzz_res = fuzzy::fuzz(name, iter, SEP).await?;
             let mut is_low = false;
             let mut is_multi_fuzz = false;
@@ -164,7 +166,7 @@ pub async fn do_script_query_strict<'b>(
             filtered.update(|script| script.miss()).await?;
             return Err(Error::ScriptIsFiltered(filtered.name.key().to_string()));
         }
-    }
+    };
 
     Err(Error::ScriptNotFound(script_query.to_string()))
 }
