@@ -4,7 +4,7 @@ use crate::config::Config;
 use crate::error::{Error, Result};
 use crate::fuzzy::{fuzz_with_multifuzz_ratio, is_prefix, FuzzResult};
 use crate::path;
-use crate::script_repo::RepoEntry;
+use crate::script_repo::{RepoEntry, Visibility};
 use crate::SEP;
 use std::cmp::Reverse;
 use structopt::StructOpt;
@@ -83,7 +83,7 @@ pub async fn handle_completion(comp: Completion) -> Result {
             new_root.sanitize_flags();
             let mut repo = init_repo(new_root.root_args, false).await?;
 
-            let iter = repo.iter_mut(false);
+            let iter = repo.iter_mut(Visibility::Normal);
             let scripts = if let Some(name) = name {
                 fuzz_arr(&name, iter).await?
             } else {
