@@ -99,6 +99,8 @@ pub fn setup_with_utils<'a>() -> MutexGuard<'a, ()> {
         }
     }
 
+    run!("ls").unwrap(); // create the home
+
     static ONCE: Once = Once::new();
     ONCE.call_once(|| {
         hyper_scripter::path::set_home(Some(&home), true).unwrap();
@@ -106,7 +108,10 @@ pub fn setup_with_utils<'a>() -> MutexGuard<'a, ()> {
         Config::set_prompt_level(Some(PromptLevel::Never));
     });
 
-    run!("alias e edit --fast").unwrap();
+    // 避免編輯器堵住整個程式
+    let mut conf = load_conf();
+    conf.editor = vec!["echo".to_owned()];
+    conf.store().unwrap();
 
     guard
 }
@@ -192,7 +197,7 @@ pub fn run_with_env<T: ToString>(env: RunEnv, args: T) -> Result<String> {
     res
 }
 
-fn get_ls(filter: Option<&str>, query: Option<&str>) -> Vec<String> {
+pub fn get_ls(filter: Option<&str>, query: Option<&str>) -> Vec<String> {
     let ls_res = run!(
         "ls {} --grouping none --plain --name {}",
         filter.map(|f| format!("-f {}", f)).unwrap_or_default(),
@@ -214,8 +219,9 @@ pub fn assert_ls_len(expect: usize, filter: Option<&str>, query: Option<&str>) {
     let res = get_ls(filter, query);
     assert_eq!(expect, res.len(), "ls {:?} 結果為 {:?}", filter, res);
 }
-pub fn assert_ls(mut expect: Vec<&str>, filter: Option<&str>, query: Option<&str>) {
-    expect.sort_unstable();
+pub fn assert_ls<T: ToString>(expect: Vec<T>, filter: Option<&str>, query: Option<&str>) {
+    let mut expect: Vec<_> = expect.into_iter().map(|s| s.to_string()).collect();
+    expect.sort();
     let mut res = get_ls(filter, query);
     res.sort();
     assert_eq!(expect, res, "ls {:?} 結果為 {:?}", filter, res);
@@ -226,6 +232,11 @@ pub fn assert_ls(mut expect: Vec<&str>, filter: Option<&str>, query: Option<&str
 pub struct ScriptTest {
     name: String,
 }
+impl<'a> ToString for &'a ScriptTest {
+    fn to_string(&self) -> String {
+        self.name.clone()
+    }
+}
 impl ScriptTest {
     pub fn get_name(&self) -> &str {
         &self.name
