@@ -7,15 +7,15 @@ use std::fs::{create_dir_all, remove_dir_all, remove_file, File};
 use std::io::prelude::*;
 use tool::*;
 
-fn test_import() {
+fn test_import(og_util_cnt: usize) {
     let tmp_dir = std::env::temp_dir();
     let dir_path = tmp_dir.join("to_be_import");
     let dir = dir_path.to_string_lossy();
     log::info!("把待匯入的腳本放進 {}", dir);
 
-    run!("e copy/test -t +innate | echo 我要留下來").unwrap();
+    run!("e copy/test -t +innate,copy | echo 我要留下來").unwrap();
     run!(
-        "e my/innate -t +innate | rm {} -rf && cp {}/tests/to_be_import {} -r",
+        "e my/innate -t +innate,my | rm {} -rf && cp {}/tests/to_be_import {} -r",
         dir,
         env!("CARGO_MANIFEST_DIR"),
         dir,
@@ -49,13 +49,13 @@ fn test_import() {
 
     assert!(check_exist(&[".gitignore"]));
 
-    assert_ls_len(16, Some("all"), None);
+    assert_ls_len(11 + og_util_cnt, Some("all"), None);
 
     run!("-f util import --namespace imported {}", dir).unwrap();
     // NOTE: 上面這行會噴一些找不到路徑的錯誤，不用緊張，是因為 `to_be_import` 裡面有些腳本被故意砍掉了
     assert_eq!(run!("-a imported/my/tes").unwrap(), "安安！紅寶石");
     run!("-f imported which").expect_err("命名空間汙染了標籤！");
-    assert_ls_len(22, Some("all"), None);
+    assert_ls_len(17 + og_util_cnt, Some("all"), None);
 
     // check content of file
     let file_path = run!("which -a imported/my/tes").unwrap();
@@ -67,7 +67,7 @@ fn test_import() {
     );
 }
 
-fn test_collect() {
+fn test_collect(og_util_cnt: usize) {
     pub fn create_all(name: &str, ext: Option<&str>, content: &str) -> String {
         let full_name = if let Some(ext) = ext {
             format!("{}.{}", name, ext)
@@ -86,16 +86,14 @@ fn test_collect() {
 
     run!("e noughty-txt.sh -T txt | echo 別收集我").unwrap();
 
+    const COLLECT_TXT: &str = "這是一個收集測試";
     let named = create_all(
         "this/is/a/collect/t.est",
         Some("rb"),
-        "puts '這是一個收集測試'",
-    );
-    let named_txt = create_all(
-        "this/is/a/txt/coll.ect/test.ggext",
-        None,
-        "這是一個文字檔收集測試",
+        &format!("puts '{COLLECT_TXT}'"),
     );
+    const SCREWED_UP_TXT: &str = "因為副檔名爛掉所以被當作文字檔";
+    let named_txt = create_all("this/is/a/txt/coll.ect/test.ggext", None, SCREWED_UP_TXT);
     create_all(".anonymous/10", Some("sh"), "echo 這是一個匿名收集測試");
     create_all(".anonymous/100", None, "這是一個匿名文字檔收集測試");
 
@@ -120,9 +118,8 @@ fn test_collect() {
 
     run!("collect").unwrap();
 
-    assert_eq!(run!("-f this {}", named).unwrap(), "這是一個收集測試");
-    assert_eq!(run!("-f is {}", named).unwrap(), "這是一個收集測試");
-    assert_eq!(run!("{}", named_txt).unwrap(), "這是一個文字檔收集測試");
+    assert_eq!(run!("{}", named).unwrap(), COLLECT_TXT);
+    assert_eq!(run!("{}", named_txt).unwrap(), SCREWED_UP_TXT);
     assert_eq!(run!(".10").unwrap(), "這是一個匿名收集測試");
     assert_eq!(run!(".100").unwrap(), "這是一個匿名文字檔收集測試");
 
@@ -134,12 +131,13 @@ fn test_collect() {
     assert_eq!(run!("-f tag youest").unwrap(), "殼已破碎");
     assert_eq!(run!("-f nameless -").unwrap(), "安安，匿名殼");
 
-    assert_ls_len(23, Some("all"), None);
+    assert_ls_len(18 + og_util_cnt, Some("all"), None);
 }
 
 #[test]
 fn test_utils() {
     let _g = setup_with_utils();
-    test_import();
-    test_collect();
+    let og_util_cnt = get_ls(Some("all"), None).len();
+    test_import(og_util_cnt);
+    test_collect(og_util_cnt);
 }
